%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Multi-stage Dantzig selector
% example.m
% reference: Multi-stage Dantzig Selector, NIPS 2010
% by: Ji Liu
% date: 09/2009
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc;
clear;
close all;

addpath('functions/');
randn('state', 2010);
rand('state', 2010);

%% generate data the design of feature matrix "X \in R^{n*m}", the observation "y \in R^m"
n = 100;
m = 200;
s = 15;                         % the sparsity number
sigma = 0.05;               % the variance of the obeservation noise

X = normalize(randn(n, m)')';
true_beta = randn(m, 1);
[a,b] = sort(abs(true_beta));
F = sort(b(end-s+1:end));
true_beta(b(1:end-s)) = 0;
true_beta(F) = (rand(s, 1)-0.5) * 20;
y = X*true_beta + randn(n,1) * sigma;       % generate y
lambda = (2*log(m))^0.5*sigma;                % the parameter in the standard Dantzig selector           

%% initialization
F0 = [];        % the initial support set
beta = [];      % the initial signal
N = s+5;         % the number iterations in multi-stage Dantzig selector

[betaList, F0List] = MultiStageDantzigSelector(F0,...
X,...
y,...
lambda,...
N,...
beta);

oracle_beta = zeros(m,1);
oracle_beta(F) = inv(X(:, F)'*X(:, F))*X(:, F)'*y;
oracle_SRA = norm(oracle_beta-true_beta) / norm(true_beta);

Mul_ROC = zeros(1,N+1);
for i = 1:N+1
    Mul_SRA(i) = norm(betaList(:,i)-true_beta) / norm(true_beta);
    [a, b] = sort(abs(betaList(:, i)));
    Mul_FSA(i) = length(intersect(F, b(end-s+1:end))) / s;
end
result{1} = [Mul_SRA, oracle_SRA];
result{2} = [Mul_FSA, 1];
SRA = result{1}
FSA = result{2}

DrawFigFinal(result, 'n=100 m=200 s=15 \sigma=0.05', s);


